import { useQuerybuilderGraph } from '@graphpolaris/shared/lib/data-access';
import React, { useMemo, useRef, useState } from 'react';
import { Handle, Position, useUpdateNodeInternals } from 'reactflow';
import { NodeAttribute, SchemaReactflowEntityNode, toHandleId } from '../../../model';
import { PillDropdown } from '../../pilldropdown/PillDropdown';
import { Button, DropdownContainer, DropdownItemContainer, DropdownTrigger, EntityPill } from '@graphpolaris/shared/lib/components';
import { ArrowDropDown, ArrowDropUp } from '@mui/icons-material';

/**
 * Component to render an entity flow element
 * @param {NodeProps} param0 The data of an entity flow element.
 */
export const QueryEntityPill = React.memo((node: SchemaReactflowEntityNode) => {
  const updateNodeInternals = useUpdateNodeInternals();
  const ref = useRef<HTMLDivElement | null>(null);

  const data = node.data;
  if (!data.leftRelationHandleId) throw new Error('EntityFlowElement: data.leftRelationHandleId is undefined');
  if (!data.rightRelationHandleId) throw new Error('EntityFlowElement: data.rightRelationHandleId is undefined');

  const graph = useQuerybuilderGraph();
  const attributeEdges = useMemo(
    () => graph.edges.filter((edge) => edge.source === node.id && !!edge?.attributes?.sourceHandleData.attributeType),
    [graph],
  );

  const [openDropdown, setOpenDropdown] = useState(false);

  return (
    <div className="w-fit h-fit nowheel" ref={ref} id="asd">
      <EntityPill
        title={
          <div className="flex flex-row justify-between items-center">
            <span>{data.name || ''}</span>
            <Button
              variantType="secondary"
              variant="ghost"
              size="2xs"
              iconComponent={openDropdown ? <ArrowDropUp /> : <ArrowDropDown />}
              onMouseDownCapture={(e) => {
                e.stopPropagation();
                e.preventDefault();
                setOpenDropdown(!openDropdown);
              }}
              className={openDropdown ? 'border-secondary-200' : ''}
            />
          </div>
        }
        withHandles="horizontal"
        handleLeft={
          <Handle
            id={toHandleId(data.leftRelationHandleId)}
            position={Position.Left}
            className={'!rounded-none !bg-transparent !w-full !h-full !border-0 !right-0 !left-0'}
            type="target"
          ></Handle>
        }
        handleRight={
          <Handle
            id={toHandleId(data.rightRelationHandleId)}
            // id="entitySourceRight"
            position={Position.Right}
            className={'!rounded-none !bg-transparent !w-full !h-full !border-0 !right-0 !left-0'}
            type="source"
          ></Handle>
        }
      >
        {data?.attributes && (
          <PillDropdown
            node={node}
            attributes={data.attributes}
            attributeEdges={attributeEdges.map((edge) => edge?.attributes)}
            open={openDropdown}
          />
        )}
      </EntityPill>
    </div>
  );
});

QueryEntityPill.displayName = 'QueryEntityPill';

export default QueryEntityPill;
